@@ -75,14 +75,6 @@ typedef enum
   GSD_TOUCHPAD_HANDEDNESS_MOUSE
 } GsdTouchpadHandedness;
 
-typedef enum
-{
-  GSD_XRANDR_BOOT_BEHAVIOUR_DO_NOTHING,
-  GSD_XRANDR_BOOT_BEHAVIOUR_CLONE,
-  GSD_XRANDR_BOOT_BEHAVIOUR_DOCK,
-  GSD_XRANDR_BOOT_BEHAVIOUR_FOLLOW_LID
-} GsdXrandrBootBehaviour;
-
 typedef enum
 {
   GSD_WACOM_ROTATION_NONE,
@@ -110,6 +102,13 @@ typedef enum
   GSD_POWER_ACTION_LOGOUT,
 } GsdPowerActionType;
 
+typedef enum
+{
+  GSD_POWER_BUTTON_ACTION_NOTHING,
+  GSD_POWER_BUTTON_ACTION_SUSPEND,
+  GSD_POWER_BUTTON_ACTION_HIBERNATE
+} GsdPowerButtonActionType;
+
 typedef enum
 {
   GSD_UPDATE_TYPE_ALL,
